@@ -118,6 +118,19 @@ class Unparser:
         self.write(" "+self.binop[t.op.__class__.__name__]+"= ")
         self.dispatch(t.value)
 
+    def _AnnAssign(self, t):
+        self.fill()
+        if not t.simple:
+            self.write("(")
+        self.dispatch(t.target)
+        if not t.simple:
+            self.write(")")
+        self.write(": ")
+        self.dispatch(t.annotation)
+        if t.value:
+            self.write(" = ")
+            self.dispatch(t.value)
+
     def _Return(self, t):
         self.fill("return")
         if t.value:
@@ -315,12 +328,12 @@ class Unparser:
         self.dispatch(t.body)
         self.leave()
 
-    def _generic_FunctionDef(self, t, async=False):
+    def _generic_FunctionDef(self, t, async_=False):
         self.write("\n")
         for deco in t.decorator_list:
             self.fill("@")
             self.dispatch(deco)
-        self.fill(("async " if async else "") + "def " + t.name + "(")
+        self.fill(("async " if async_ else "") + "def " + t.name + "(")
         self.dispatch(t.args)
         self.write(")")
         if getattr(t, "returns", False):
@@ -334,10 +347,10 @@ class Unparser:
         self._generic_FunctionDef(t)
 
     def _AsyncFunctionDef(self, t):
-        self._generic_FunctionDef(t, async=True)
+        self._generic_FunctionDef(t, async_=True)
 
-    def _generic_For(self, t, async=False):
-        self.fill("async for " if async else "for ")
+    def _generic_For(self, t, async_=False):
+        self.fill("async for " if async_ else "for ")
         self.dispatch(t.target)
         self.write(" in ")
         self.dispatch(t.iter)
@@ -354,7 +367,7 @@ class Unparser:
         self._generic_For(t)
 
     def _AsyncFor(self, t):
-        self._generic_For(t, async=True)
+        self._generic_For(t, async_=True)
 
     def _If(self, t):
         self.fill("if ")
@@ -390,8 +403,8 @@ class Unparser:
             self.dispatch(t.orelse)
             self.leave()
 
-    def _generic_With(self, t, async=False):
-        self.fill("async with " if async else "with ")
+    def _generic_With(self, t, async_=False):
+        self.fill("async with " if async_ else "with ")
         if hasattr(t, 'items'):
             interleave(lambda: self.write(", "), self.dispatch, t.items)
         else:
@@ -407,7 +420,7 @@ class Unparser:
         self._generic_With(t)
 
     def _AsyncWith(self, t):
-        self._generic_With(t, async=True)
+        self._generic_With(t, async_=True)
 
     # expr
     def _Bytes(self, t):
@@ -429,6 +442,52 @@ class Unparser:
             else:
                 assert False, "shouldn't get here"
 
+    def _JoinedStr(self, t):
+        # JoinedStr(expr* values)
+        self.write("f")
+        string = StringIO()
+        self._fstring_JoinedStr(t, string.write)
+        self.write(repr(string.getvalue()))
+
+    def _FormattedValue(self, t):
+        # FormattedValue(expr value, int? conversion, expr? format_spec)
+        self.write("f")
+        string = StringIO()
+        self._fstring_JoinedStr(t, string.write)
+        self.write(repr(string.getvalue()))
+
+    def _fstring_JoinedStr(self, t, write):
+        for value in t.values:
+            meth = getattr(self, "_fstring_" + type(value).__name__)
+            meth(value, write)
+
+    def _fstring_Str(self, t, write):
+        value = t.s.replace("{", "{{").replace("}", "}}")
+        write(value)
+
+    def _fstring_Constant(self, t, write):
+        assert isinstance(t.value, str)
+        value = t.value.replace("{", "{{").replace("}", "}}")
+        write(value)
+
+    def _fstring_FormattedValue(self, t, write):
+        write("{")
+        expr = StringIO()
+        Unparser(t.value, expr)
+        expr = expr.getvalue().rstrip("\n")
+        if expr.startswith("{"):
+            write(" ")  # Separate pair of opening brackets as "{ {"
+        write(expr)
+        if t.conversion != -1:
+            conversion = chr(t.conversion)
+            assert conversion in "sra"
+            write("!{conversion}".format(conversion=conversion))
+        if t.format_spec:
+            write(":")
+            meth = getattr(self, "_fstring_" + type(t.format_spec).__name__)
+            meth(t.format_spec, write)
+        write("}")
+
     def _Name(self, t):
         self.write(t.id)
 
@@ -491,6 +550,8 @@ class Unparser:
         self.write("}")
 
     def _comprehension(self, t):
+        if getattr(t, 'is_async', False):
+            self.write(" async")
         self.write(" for ")
         self.dispatch(t.target)
         self.write(" in ")
@@ -518,11 +579,19 @@ class Unparser:
         self.write("{")
         def write_pair(pair):
             (k, v) = pair
-            self.dispatch(k)
-            self.write(": ")
-            self.dispatch(v)
-        interleave(lambda: self.write(", "), write_pair, zip(t.keys, t.values))
-        self.write("}")
+            if k is None:
+                self.write('**')
+                self.dispatch(v)
+            else:
+                self.dispatch(k)
+                self.write(": ")
+                self.dispatch(v)
+            self.write(",")
+        self._indent +=1
+        self.fill("")
+        interleave(lambda: self.fill(""), write_pair, zip(t.keys, t.values))
+        self._indent -=1
+        self.fill("}")
 
     def _Tuple(self, t):
         self.write("(")
